@@ -5,7 +5,6 @@ use std::slice::Iter;
 use std::time::Instant;
 
 use aho_corasick::AhoCorasick;
-use bincode::Options;
 #[cfg(feature = "logging")]
 use log::*;
 use regex_automata::meta::Regex;
@@ -165,9 +164,11 @@ impl Rules {
         let start = Instant::now();
 
         // Skip the magic and deserialize the remaining data.
-        let mut rules = bincode::DefaultOptions::new()
-            .with_varint_encoding()
-            .deserialize::<Self>(&bytes[magic.len()..])?;
+        let (mut rules, _len): (Self, usize) =
+            bincode::serde::decode_from_slice(
+                &bytes[magic.len()..],
+                bincode::config::standard(),
+            )?;
 
         #[cfg(feature = "logging")]
         info!("Deserialization time: {:?}", Instant::elapsed(&start));
@@ -215,10 +216,13 @@ impl Rules {
         // Write file header.
         writer.write_all(b"YARA-X")?;
 
-        // Serialize rules.
-        Ok(bincode::DefaultOptions::new()
-            .with_varint_encoding()
-            .serialize_into(writer, self)?)
+        bincode::serde::encode_into_std_write(
+            self,
+            &mut writer,
+            bincode::config::standard(),
+        )?;
+
+        Ok(())
     }
 
     /// Deserializes the rules from a `reader`.
@@ -445,9 +449,13 @@ impl Rules {
 
     #[inline]
     pub(crate) fn globals(&self) -> types::Struct {
-        bincode::DefaultOptions::new()
-            .deserialize::<types::Struct>(self.serialized_globals.as_slice())
-            .expect("error deserializing global variables")
+        let (globals, _): (types::Struct, usize) =
+            bincode::serde::decode_from_slice(
+                self.serialized_globals.as_slice(),
+                bincode::config::standard(),
+            )
+            .expect("error deserializing global variables");
+        globals
     }
 
     #[inline]
