@@ -23,17 +23,19 @@ pub enum Warning {
     DuplicateImport(Box<DuplicateImport>),
     IgnoredModule(Box<IgnoredModule>),
     IgnoredRule(Box<IgnoredRule>),
+    InvalidMetadata(Box<InvalidMetadata>),
+    InvalidRuleName(Box<InvalidRuleName>),
+    InvalidTag(Box<InvalidTag>),
     InvariantBooleanExpression(Box<InvariantBooleanExpression>),
+    MissingMetadata(Box<MissingMetadata>),
     NonBooleanAsBoolean(Box<NonBooleanAsBoolean>),
     PotentiallySlowLoop(Box<PotentiallySlowLoop>),
     PotentiallyUnsatisfiableExpression(Box<PotentiallyUnsatisfiableExpression>),
     RedundantCaseModifier(Box<RedundantCaseModifier>),
     SlowPattern(Box<SlowPattern>),
     TextPatternAsHex(Box<TextPatternAsHex>),
-    InvalidMetadata(Box<InvalidMetadata>),
-    MissingMetadata(Box<MissingMetadata>),
-    InvalidRuleName(Box<InvalidRuleName>),
-    InvalidTag(Box<InvalidTag>),
+    TooManyIterations(Box<TooManyIterations>),
+    UnsatisfiableExpression(Box<UnsatisfiableExpression>),
     UnknownTag(Box<UnknownTag>),
 }
 
@@ -102,7 +104,7 @@ impl ConsecutiveJumps {
     title = "potentially slow loop",
 )]
 #[label(
-"this range can be very large",
+    "this range can be very large",
     loc
 )]
 pub struct PotentiallySlowLoop {
@@ -149,6 +151,44 @@ pub struct PotentiallyUnsatisfiableExpression {
     at_loc: CodeLoc,
 }
 
+/// A boolean expression can't be satisfied.
+///
+/// ## Example
+///
+/// ```text
+/// warning[unsatisfiable_expr]: unsatisfiable expression
+/// --> test.yar:6:34
+/// |
+/// 6 | rule x { condition: "AD" == hash.sha256(0,filesize) }
+/// |                     ----         ------------------ this is a lowercase string
+/// |                     |
+/// |                     this contains uppercase characters
+/// |
+/// = note: a lowercase strings can't be equal to a string containing uppercase characters
+#[derive(ErrorStruct, Debug, PartialEq, Eq)]
+#[associated_enum(Warning)]
+#[warning(
+    code = "unsatisfiable_expr",
+    title = "unsatisfiable expression"
+)]
+#[label(
+    "{label_1}",
+    loc_1
+)]
+#[label(
+    "{label_2}",
+    loc_2
+)]
+#[footer(note)]
+pub struct UnsatisfiableExpression {
+    report: Report,
+    label_1: String,
+    label_2: String,
+    loc_1: CodeLoc,
+    loc_2: CodeLoc,
+    note: Option<String>,
+}
+
 
 /// A boolean expression always has the same value.
 ///
@@ -557,8 +597,41 @@ pub struct InvalidRuleName {
     regex: String,
 }
 
+/// A loop or nested loops have a total number of iterations exceeding a
+/// predefined threshold.
+///
+/// This warning indicates that a rule contains a `for` loop, or a set of nested
+/// `for` loops, that may be very slow because the total number of iterations
+/// is very large.
+///
+/// # Example
+///
+/// ```text
+/// warning[too_many_iterations]: loop has too many iterations
+///  --> test.yar:1:20
+///   |
+/// 1 | rule t { condition: for any i in (0..1000) : ( for any j in (0..1000) : ( true ) ) }
+///   |                    -------------------------------------------------------------- this loop iterates 1000000 times, which may be slow
+///   |
+/// ```
+#[derive(ErrorStruct, Debug, PartialEq, Eq)]
+#[associated_enum(Warning)]
+#[warning(
+    code = "too_many_iterations",
+    title = "loop has too many iterations",
+)]
+#[label(
+    "this loop iterates {iterations} times, which may be slow",
+    loc
+)]
+pub struct TooManyIterations {
+    report: Report,
+    iterations: i64,
+    loc: CodeLoc,
+}
+
 /// Unknown tag. This is only used if the compiler is configured to check
-/// for required tags (see:  [`crate::linters::Tags`]).
+/// for required tags (see: [`crate::linters::Tags`]).
 ///
 /// ## Example
 ///
@@ -569,7 +642,7 @@ pub struct InvalidRuleName {
 /// 1 | rule a : foo {
 ///   |          --- tag `foo` not in allowed list
 ///   |
-///   = note: Allowed tags: test, bar
+///   = note: allowed tags: test, bar
 /// ```
 #[derive(ErrorStruct, Clone, Debug, PartialEq, Eq)]
 #[associated_enum(Warning)]
