@@ -34,7 +34,7 @@ use crate::modules::{Module, BUILTIN_MODULES};
 use crate::scanner::matches::PatternMatches;
 use crate::types::{Struct, TypeValue};
 use crate::variables::VariableError;
-use crate::wasm::{ENGINE, MATCHING_RULES_BITMAP_BASE};
+use crate::wasm::MATCHING_RULES_BITMAP_BASE;
 use crate::{modules, wasm, Variable};
 
 pub(crate) use crate::scanner::context::RuntimeObject;
@@ -623,7 +623,7 @@ impl<'r> Scanner<'r> {
             INIT_HEARTBEAT.call_once(|| {
                 thread::spawn(|| loop {
                     thread::sleep(Duration::from_secs(1));
-                    ENGINE.increment_epoch();
+                    crate::wasm::ENGINE.increment_epoch();
                     HEARTBEAT_COUNTER
                         .fetch_update(
                             Ordering::SeqCst,
