@@ -80,9 +80,9 @@ See the [`lookup_field`] function.
 use std::any::{type_name, TypeId};
 use std::mem;
 use std::rc::Rc;
+use std::sync::LazyLock;
 
 use bstr::{BString, ByteSlice};
-use lazy_static::lazy_static;
 use linkme::distributed_slice;
 use rustc_hash::FxHashMap;
 use smallvec::{smallvec, SmallVec};
@@ -98,7 +98,9 @@ use crate::scanner::{RuntimeObjectHandle, ScanContext, ScanError};
 use crate::types::{
     Array, Func, FuncSignature, Map, Struct, TypeValue, Value,
 };
+use crate::wasm;
 use crate::wasm::string::RuntimeString;
+use crate::wasm::string::String as _;
 
 pub(crate) mod builder;
 pub(crate) mod string;
@@ -464,7 +466,7 @@ impl WasmResult for bool {
     }
 }
 
-impl WasmResult for RuntimeString {
+impl<S: wasm::string::String> WasmResult for S {
     fn values(self, ctx: &mut ScanContext) -> WasmResultArray<ValRaw> {
         smallvec![ValRaw::i64(self.into_wasm_with_ctx(ctx))]
     }
@@ -705,49 +707,48 @@ pub(crate) struct WasmSymbols {
     pub f64_tmp: walrus::LocalId,
 }
 
-lazy_static! {
-    pub(crate) static ref CONFIG: Config = {
-        let mut config = Config::default();
-        // Wasmtime produces a nasty warning when linked against musl. The
-        // warning can be fixed by disabling native unwind information.
-        //
-        // More details:
-        //
-        // https://github.com/bytecodealliance/wasmtime/issues/8897
-        // https://github.com/VirusTotal/yara-x/issues/181
-        //
-        #[cfg(target_env = "musl")]
-        config.native_unwind_info(false);
+pub(crate) static CONFIG: LazyLock<Config> = LazyLock::new(|| {
+    let mut config = Config::default();
+    // Wasmtime produces a nasty warning when linked against musl. The
+    // warning can be fixed by disabling native unwind information.
+    //
+    // More details:
+    //
+    // https://github.com/bytecodealliance/wasmtime/issues/8897
+    // https://github.com/VirusTotal/yara-x/issues/181
+    //
+    #[cfg(target_env = "musl")]
+    config.native_unwind_info(false);
 
-        config.cranelift_opt_level(wasmtime::OptLevel::SpeedAndSize);
-        config.epoch_interruption(true);
+    config.cranelift_opt_level(wasmtime::OptLevel::SpeedAndSize);
+    config.epoch_interruption(true);
 
-        // 16MB should be enough for each WASM module. Each module needs a
-        // fixed amount of memory that is only a few KB long, plus a variable
-        // amount that depends on the number of rules and patterns (1 bit per
-        // rule and 1 bit per pattern). With 16MB there's enough space for
-        // millions of rules and patterns. By default, this is 4GB in 64-bits
-        // systems, which causes a reservation of 4GB of virtual address space
-        // (not physical RAM) per module (and therefore per Scanner). In some
-        // scenarios where virtual address space is limited (i.e: Docker
-        // instances) this is problematic. See:
-        // https://github.com/VirusTotal/yara-x/issues/292
-        config.memory_reservation(0x1000000);
+    // 16MB should be enough for each WASM module. Each module needs a
+    // fixed amount of memory that is only a few KB long, plus a variable
+    // amount that depends on the number of rules and patterns (1 bit per
+    // rule and 1 bit per pattern). With 16MB there's enough space for
+    // millions of rules and patterns. By default, this is 4GB in 64-bits
+    // systems, which causes a reservation of 4GB of virtual address space
+    // (not physical RAM) per module (and therefore per Scanner). In some
+    // scenarios where virtual address space is limited (i.e: Docker
+    // instances) this is problematic. See:
+    // https://github.com/VirusTotal/yara-x/issues/292
+    config.memory_reservation(0x1000000);
 
-        // WASM memory won't grow, there's no need to allocate space for
-        // future grow.
-        config.memory_reservation_for_growth(0);
+    // WASM memory won't grow, there's no need to allocate space for
+    // future grow.
+    config.memory_reservation_for_growth(0);
 
-        // As the memory can't grow, it won't move. By explicitly indicating
-        // this, modules can be compiled with static knowledge the base pointer
-        // of linear memory never changes to enable optimizations.
-        config.memory_may_move(false);
+    // As the memory can't grow, it won't move. By explicitly indicating
+    // this, modules can be compiled with static knowledge the base pointer
+    // of linear memory never changes to enable optimizations.
+    config.memory_may_move(false);
 
-        config
-    };
-    pub(crate) static ref ENGINE: Engine = Engine::new(&CONFIG).unwrap();
-    pub(crate) static ref LINKER: Linker<ScanContext<'static>> = new_linker();
-}
+    config
+});
+
+pub(crate) static ENGINE: LazyLock<Engine> =
+    LazyLock::new(|| Engine::new(&CONFIG).unwrap());
 
 pub(crate) fn new_linker<'r>() -> Linker<ScanContext<'r>> {
     let mut linker = Linker::<ScanContext<'r>>::new(&ENGINE);
@@ -1035,9 +1036,13 @@ pub(crate) fn lookup_string(
     num_lookup_indexes: i32,
 ) -> Option<RuntimeString> {
     match lookup_field(caller, structure, num_lookup_indexes) {
-        TypeValue::String(Value::Var(s)) => Some(RuntimeString::Rc(s)),
-        TypeValue::String(Value::Const(s)) => Some(RuntimeString::Rc(s)),
-        TypeValue::String(Value::Unknown) => None,
+        TypeValue::String { value: Value::Var(s), .. } => {
+            Some(RuntimeString::Rc(s))
+        }
+        TypeValue::String { value: Value::Const(s), .. } => {
+            Some(RuntimeString::Rc(s))
+        }
+        TypeValue::String { value: Value::Unknown, .. } => None,
         _ => unreachable!(),
     }
 }
@@ -1069,7 +1074,7 @@ macro_rules! gen_lookup_fn {
             structure: Option<Rc<Struct>>,
             num_lookup_indexes: i32,
         ) -> Option<$return_type> {
-            if let $type(value) =
+            if let $type { value } =
                 lookup_field(caller, structure, num_lookup_indexes)
             {
                 value.extract().cloned()
